#include <assert.h>
#include <math.h>
#include <stddef.h>

#include "optik.h"
#include "tools.h"

#define pi 3.14159265358979323846264338327950288419716939937510582097494459L
#define deg_rad(deg) ((deg) * pi / 180.0)
#define rad_deg(rad) ((rad) * 180.0 / pi)

BOOL optik_abbm_anhand_g(double f, double g, double* beta) {

	BOOL params_ok = (f > 0 && g > f && beta != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*beta = f / (g - f);

	return TRUE;
}

BOOL optik_bildweite_anhand_abbm(double f, double beta, double* b) {

	BOOL params_ok = (f > 0 && beta > 0 && b != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*b = (beta + 1.0) * f;

	return TRUE;
}

BOOL optik_bildweite_anhand_g(double f, double g, double* b) {

	BOOL params_ok = (f > 0 && g > f && b != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*b = (f * g) / (g - f);

	return TRUE;
}

BOOL optik_g_anhand_abbm(double f, double beta, double* g) {

	BOOL params_ok = (f > 0 && beta > 0 && g != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*g = (1.0 / beta + 1.0) * f;

	return TRUE;
}

BOOL optik_g_anhand_bildweite(double f, double b, double* g) {

	BOOL params_ok = (f > 0 && b > 0 && b != f && g != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*g = (f * b) / (b - f);

	return TRUE;
}

BOOL optik_foerderliche_blende(double z, double beta, double lambda, double* k_f) {

	BOOL params_ok = (z > 0 && beta > 0 && lambda > 0 && k_f != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*k_f = z / (1.22 * lambda * (beta + 1.0));

	return TRUE;
}

BOOL optik_effektive_blende(double k, double beta, double* k_e) {

	BOOL params_ok = (k > 0 && beta > 0 && k_e != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*k_e= k * (beta + 1.0);

	return TRUE;
}

BOOL optik_effektive_blende_p(double k, double beta, double beta_p, double* k_e) {

	BOOL params_ok = (k > 0 && beta > 0 && beta_p > 0 && k_e != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*k_e = k * (beta / beta_p + 1.0);

	return TRUE;
}

BOOL optik_effektive_blende_p_r(double k, double beta, double beta_p, double* k_e) {

	BOOL params_ok = (k > 0 && beta > 0 && beta_p > 0 && k_e != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*k_e = k * (beta * beta_p + 1.0);

	return TRUE;
}

BOOL optik_pupillen_masstab(double ap, double ep, double* beta_p) {

	BOOL params_ok = (ap > 0 && ep > 0 && beta_p != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*beta_p = ap / ep;

	return TRUE;
}

BOOL optik_blende_pup_masstab(double k, double beta_p, double* k_neu) {

	BOOL params_ok = (k > 0 && beta_p > 0 && k_neu != NULL);

	if (params_ok) return FALSE;

	*k_neu = k / beta_p;

	return TRUE;
}

BOOL optik_t_faktor_anhand_bildweite(double b, double f, double* faktor) {

	BOOL params_ok = (b > 0 && f > 0 && faktor != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*faktor = tools_quad(b / f);

	return TRUE;
}

BOOL optik_k_stufen_anhand_t_faktor(double t_faktor, double* k_stufen) {

	BOOL params_ok = (t_faktor > 0.0 && k_stufen != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*k_stufen = log(t_faktor) / log(2);

	return TRUE;
}

BOOL optik_t_faktor_anhand_abbm(double beta, double f, double* faktor) {

	BOOL params_ok = (beta > 0 && f > 0 && faktor != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*faktor = tools_quad(beta + 1.0);

	return TRUE;
}

BOOL optik_t_faktor_anhand_abbm_p(double beta, double beta_p, double* faktor) {

	BOOL params_ok = (beta > 0 && beta_p > 0 && faktor != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*faktor = tools_quad(beta / beta_p + 1.0);

	return TRUE;
}

BOOL optik_t_faktor_anhand_abbm_p_r(double beta, double beta_p, double* faktor) {

	BOOL params_ok = (beta > 0 && beta_p > 0 && faktor != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*faktor = tools_quad(beta * beta_p + 1.0);

	return TRUE;
}

BOOL optik_loch_durchmesser_opt(double lambda, double b, double g, double* d) {

	BOOL params_ok = (lambda > 0 && b > 0 && g > 0 && d != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*d = (double) (sqrt((2.56 * lambda) / (1.0 / g + 1.0 / b)));

	return TRUE;
}

BOOL optik_dioptrien(double f, double* diop) {

	BOOL params_ok = (f > 0 && diop != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*diop = 1.0 / f;

	return TRUE;
}

BOOL optik_brennweite_anhand_dioptrien(double diop, double* f) {

	BOOL params_ok = (diop > 0 && f != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*f = 1 / diop;

	return TRUE;
}

BOOL optik_abbm_nahlinse_u(double f, double diop, double* beta) {

	BOOL params_ok = (f > 0 && diop > 0 && beta != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*beta = f * diop;

	return TRUE;
}

BOOL optik_abbm_nahlinse(double f, double diop, double g, double* beta) {

	BOOL params_ok = (f > 0 && diop > 0 && g > f && beta != NULL);
	double fNl = 0.0;

	assert(params_ok); if (!params_ok) return FALSE;

	optik_brennweite_anhand_dioptrien(diop, &fNl);
	*beta = (f * (g + fNl)) / (fNl * (g - f));

	return TRUE;
}

BOOL optik_g_nahlinse_u(double diop, double* g) {

	BOOL params_ok = (diop > 0 && g != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	optik_brennweite_anhand_dioptrien(diop, g);

	return TRUE;
}

BOOL optik_g_nahlinse(double diop, double g, double* g_neu) {

	BOOL params_ok = (diop > 0 && g > 0 && g_neu != NULL);
	double fNl = 0.0;

	assert(params_ok); if (!params_ok) return FALSE;

	optik_brennweite_anhand_dioptrien(diop, &fNl);
	*g_neu = (fNl * g) / (g + fNl); /* fN1 > 0 */

	return TRUE;
}

BOOL optik_breite_bildwand(double f, double g, double l, double* breite) {

	BOOL params_ok = (f > 0 && g > f && l > 0 && breite != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*breite = (g * l) / f;

	return TRUE;
}

BOOL optik_blende_anhand_lichtmengen_faktor(double k, double delta_li, double* k_neu) {

	BOOL params_ok = (k > 0 && delta_li > 0 && k_neu != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*k_neu = k * sqrt(delta_li);

	return TRUE;
}

BOOL optik_mired(double kelvin, double* mired)
{
	BOOL params_ok = (kelvin > 0 && mired != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*mired = 1000000.0 / kelvin;

	return TRUE;
}

BOOL optik_kelvin(double mired, double* kelvin) {

	BOOL params_ok = (mired > 0 && kelvin != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*kelvin = 1000000.0 / mired;

	return TRUE;
}

BOOL optik_konversions_filter(double mired_film, double mired_lichtq, double* mired_konv) {

	BOOL params_ok = (mired_film > 0 && mired_lichtq > 0 && mired_konv != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*mired_konv = mired_film - mired_lichtq;

	return TRUE;
}

BOOL optik_bildwinkel(double f, double bildlaenge, double* bildwinkel) {

	BOOL params_ok = (f > 0 && bildlaenge > 0 && bildwinkel != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*bildwinkel = 2.0 * atan(bildlaenge / (2.0 * f));

	*bildwinkel = rad_deg(*bildwinkel);

	return TRUE;
}

BOOL optik_ev(double k, double t, double* ev) {

	BOOL params_ok = (k > 0 && t > 0 && ev != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*ev = (2.0 * log(k) - log(t)) / log(2.0);

	return TRUE;
}

BOOL optik_g_anhand_entfernungs_einstellung(double e, double d, double f, double* g) {

	BOOL params_ok = (e > 0 && e > d && f > 0 && g != NULL);

	params_ok = params_ok && (tools_quad((e - d) / 2.0) >= (f * (e - d)));

	assert(params_ok); if (!params_ok) return FALSE;

	*g = (e - d) / 2.0 + sqrt(tools_quad((e - d) / 2.0) - f * (e - d));

	return TRUE;
}
